package com.diagbot.client.hystrix;

import com.diagbot.client.AiptServiceClient;
import com.diagbot.client.bean.CalculateData;
import com.diagbot.client.bean.ConceptBaseVO;
import com.diagbot.client.bean.DisTypeVO;
import com.diagbot.client.bean.GdbResponse;
import com.diagbot.client.bean.ResponseData;
import com.diagbot.client.bean.SearchData;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.ConceptIntroduceDTO;
import com.diagbot.dto.DeptInfoDTO;
import com.diagbot.dto.DisScaleDTO;
import com.diagbot.dto.DisTypeDTO;
import com.diagbot.dto.DiseaseIcdDTO;
import com.diagbot.dto.EvaluationDTO;
import com.diagbot.dto.GetTopPatientInfoDTO;
import com.diagbot.dto.IndexConfigDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.ScaleContent;
import com.diagbot.vo.ConceptIntroduceVO;
import com.diagbot.vo.ConceptSearchVO;
import com.diagbot.vo.ConceptUsualVO;
import com.diagbot.vo.DeptInfoVO;
import com.diagbot.vo.DisScaleVO;
import com.diagbot.vo.EvaluationdiseaseNameVO;
import com.diagbot.vo.GetDiseaseIcdVO;
import com.diagbot.vo.GetTopPatientInfoVO;
import com.diagbot.vo.IndexConfigdiseaseNameVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * @Description:调用中间层服务
 * @Author: Mark Huang
 * @time: 2019/03/25 19:15
 */
@Component
@Slf4j
public class AiptServiceHystrix implements AiptServiceClient {
    @Override
    public RespDTO<ResponseData> aiptData(SearchData searchData) {
        log.error("【hystrix】调用{}异常", "aiptData");
        return null;
    }

    @Override
    public RespDTO<Map<String, Object>> scale(@RequestBody SearchData searchData) {
        log.error("【hystrix】调用{}异常", "scale");
        return null;
    }

    @Override
    public RespDTO<List<Map<String, Object>>> symptomFeaturePageData(@RequestParam("text") String text) {
        log.error("【hystrix】调用{}异常", "symptomFeaturePageData");
        return null;
    }

    @Override
    public RespDTO<GdbResponse> highRisk(@RequestBody SearchData searchData) {
        log.error("【hystrix】调用{}异常", "highRisk");
        return null;
    }

    @Override
    public RespDTO<Map<String, Object>> scaleCalc(@RequestBody CalculateData calculateData) {
        log.error("【hystrix】调用{}异常", "scaleCalc");
        return null;
    }

    @Override
    public RespDTO<List<DisScaleDTO>> getList(DisScaleVO scaleVO) {
        log.error("【hystrix】调用{}异常", "getList");
        return null;
    }

    @Override
    public RespDTO<DisTypeDTO> getDisType() {
        log.error("【hystrix】调用{}异常", "getDisType");
        return null;
    }

    @Override
    public RespDTO<List<ScaleContent>> getScale(SearchData searchData) {
        log.error("【hystrix】调用{}异常", "getScale");
        return null;
    }

    @Override
    public RespDTO<ConceptIntroduceDTO> getConceptDetail(@Valid @RequestBody ConceptIntroduceVO conceptIntroduceVO) {
        log.error("【hystrix】调用{}异常", "getConceptDetail");
        return null;
    }

    @Override
    public RespDTO<List<ConceptBaseDTO>> getConceptUsual(@RequestBody @Valid ConceptUsualVO conceptUsualVO) {
        log.error("【hystrix】调用{}异常", "getConceptUsual");
        return null;
    }

    @Override
    public RespDTO<DeptInfoDTO> getDeptInfo(@Valid @RequestBody DeptInfoVO deptInfoVO) {
        log.error("【hystrix】调用{}异常", "getDeptInfo");
        return null;
    }

    /**
     * 根据医院编号疾病概念名称获取各医院对应的诊断icd
     *
     * @param getDiseaseIcdVO
     * @return
     */
    @Override
    public RespDTO<List<DiseaseIcdDTO>> getDiseaseIcds(@RequestBody GetDiseaseIcdVO getDiseaseIcdVO) {
        log.error("【hystrix】调用{}异常", "getDiseaseIcds");
        return null;
    }

    @Override
    public RespDTO<GetTopPatientInfoDTO> getTopPatientInfo(GetTopPatientInfoVO getTopPatientInfoVO) {
        log.error("【hystrix】调用{}异常", "getTopPatientInfo");
        return null;
    }

    @Override
    public RespDTO<List<EvaluationDTO>> getEvaluationModules(@Valid EvaluationdiseaseNameVO evaluationdiseaseNameVO) {
        log.error("【hystrix】调用{}异常", "getEvaluationModules");
        return null;
    }

    @Override
    public RespDTO<List<IndexConfigDTO>> getindexConfigs(IndexConfigdiseaseNameVO indexConfigdiseaseNameVO) {
        log.error("【hystrix】调用{}异常", "getindexConfigs");
        return null;
    }

    /**
     * 根据名称和词性获取概念Map
     *
     * @param conceptSearchVO 搜索参数
     * @return 术语和术语id Map
     */
    @Override
    public RespDTO<Map<String, Long>> getConceptMap(@RequestBody ConceptSearchVO conceptSearchVO) {
        log.error("【hystrix】调用{}异常", "getConceptMap");
        return null;
    }

    /**
     * 获取治疗方案
     *
     * @param searchData
     * @return
     */
    @Override
    public RespDTO<Map<String, Object>> getTreatment(@RequestBody SearchData searchData) {
        log.error("【hystrix】调用{}异常", "getTreatment");
        return null;
    }

    /**
     * 是否有提示信息
     *
     * @param conceptBaseVO
     * @return
     */
    @Override
    public RespDTO<Boolean> existConceptDetail(@Valid @RequestBody ConceptBaseVO conceptBaseVO) {
        log.error("【hystrix】调用{}异常", "existConceptDetail");
        return null;
    }

    /**
     * 验证诊断类型
     *
     * @param disTypeVO
     * @return
     */
    @Override
    public RespDTO<Boolean> verifyDisType(@Valid @RequestBody DisTypeVO disTypeVO) {
        log.error("【hystrix】调用{}异常", "verifyDisType");
        return null;
    }
}